<?php
if(!defined('MYBB_ADMIN_DIR')){
	if(!isset($config['admin_dir'])){
		$config['admin_dir'] = "admin";
	}
	define('MYBB_ADMIN_DIR', MYBB_ROOT.$config['admin_dir'].'/');
}

function ListFiles($dir) {
	if($dh = opendir($dir)) {
		$files = Array();
		$inner_files = Array();
		while($file = readdir($dh)) {
			if($file != "." && $file != ".." && $file[0] != '.') {
				if(is_dir($dir . "/" . $file)) {
					$inner_files = ListFiles($dir . "/" . $file);
					if(is_array($inner_files))
						$files = array_merge($files, $inner_files); 
				} else {
					array_push($files, str_replace("//","/",$dir . "/" . $file));
				}
			}
		}
		closedir($dh);
		return $files;
	}
}
function task_ultibackup($task){
	global $mybb, $db, $config, $lang;
	static $contents;
	$files = ListFiles(MYBB_ROOT);

	//Create temp SQL backup file to add to zip
	$sqfile = MYBB_ADMIN_DIR."ultibackup/temp/schema";
	if(function_exists('gzopen')){
		$fp = gzopen($sqfile.'.sql.gz', 'w9');
		$sqfileex = '.sql.gz';
	}else{
		$fp = fopen($sqfile.'.sql', 'w');
		$sqfileex = '.sql';
	}

	$tables = $db->list_tables($config['database']['database'], $config['database']['table_prefix']);
	$time = date('dS F Y \a\t H:i', TIME_NOW);
	$header = "-- MyBB Database Backup\n-- Generated: {$time}\n-- -------------------------------------\n\n";
	$contents = $header;
	foreach($tables as $table){
		$table = str_ireplace(TABLE_PREFIX,"",$table);
		$field_list = array();
		$fields_array = $db->show_fields_from($table);
		foreach($fields_array as $field){
			$field_list[] = $field['Field'];
		}
		$fields = "`".implode("`,`", $field_list)."`";
		$structure = $db->show_create_table($table).";\n";
		$contents .= $structure;
		$query = $db->simple_select($table);
		while($row = $db->fetch_array($query)){
			$insert = "INSERT INTO {$table} ($fields) VALUES (";
			$comma = '';
			foreach($field_list as $field)
			{
				if(!isset($row[$field]) || is_null($row[$field]))
				{
					$insert .= $comma."NULL";
				}
				else
				{
					$insert .= $comma."'".$db->escape_string($row[$field])."'";
				}
				$comma = ',';
			}
			$insert .= ");\n";
			$contents .= $insert;
		}
	}
	$db->set_table_prefix(TABLE_PREFIX);
	if(function_exists('gzopen')){
		gzwrite($fp, $contents);
		gzclose($fp);
	}else{
		fwrite($fp, $contents);
		fclose($fp);
	}
	//create a zip file
	$filename = 'backup_'.substr(md5($mybb->user['uid'].TIME_NOW), 0, 10).random_str(54).'.zip';
	$destination = MYBB_ADMIN_DIR."ultibackup/".$filename;
	$backup = new ZipArchive();
	if($backup->open($destination,ZIPARCHIVE::CREATE)){
		//add the files to the zip
		foreach($files as $file){
			$backup->addFile($file,str_ireplace(MYBB_ROOT,"files/",$file));
		}
		$backup->addFile($sqfile.$sqfileex,'db/schema'.$sqfileex);
		$backup->close();
		unlink($sqfile.$sqfileex);
	}
	if($mybb->settings['ultibackup_ftp'] == 1){
		$hostArr = explode(":",$mybb->settings['ultibackup_ftphost']);
		$host = $hostArr[0];
		$port = $hostArr[1];
		if($port == ""){
			$port = 21;
		}
		echo "Attempting to connect to FTP server...<Br />";
		$ftp = ftp_connect($host,$port);
		if($ftp){
			if(ftp_login($ftp, $mybb->settings['ultibackup_ftpuser'],$mybb->settings['ultibackup_ftppass'])){
				$file = $mybb->settings['ultibackup_ftppath'].$filename;
				if(ftp_put($ftp, $file, $destination, FTP_BINARY)){
					ftp_close($ftp);
					return true;
				}else{
					return false;
				}
			}else{
				return false;
			}
		}else{
			return false;
		}
	}
	if($mybb->settings['ultibackup_email'] == 1){
		$from = '"'.$mybb->settings['bbname'].'" <'.$mybb->settings['adminemail'].'>';
		$to = $mybb->settings['ultibackup_emailaddr'];
		$subject = "Backup of your forum.";
		$fileatt = $destination;
		$mimetype = 'application/zip';
		$fileattname = "backup_".TIME_NOW.".zip";
		$headers = "From: ".$from;
		$file = fopen($fileatt, 'rb');
		$data = fread($file, $filesize($fileatt));
		fclose($file);
		$semi_rand = md5(time());
		$mime_boundary = "==Multipart_Boundary_x{$semi_rand}x";
		$headers .= "\nMIME_Version: 1.0\n".
		"Content-Type: multiparty/mixed;\n".
		" boundary=\"{$mime_boundary}\"";
		$message = "This is a backup of your MyBB forum.\n\n" .
		"-{$mime_boundary}\n".
		"Content-Type: text/plain; charset=\"iso-8859-1\n" .
		"Content-Transfer-Encoding: 7bit\n\n" .
		$message .= "\n\n";
		$data = chunk_split(base64_encode($data));
		$message .= "-{$mime_boundary}\n".
		"Content-Type: {$mimetype};\n".
		" name=\"{$fileattname}\"\n".
		"Content-Disposition: attachment;\n".
		" filename=\"{$fileattname}\"\n".
		"Content-Transfer-Encoding: base64\n\n".
		$data . "\n\n".
		"-{$mime_boundary}-\n";
		if(mail($to, $subject, $message, $headers)){
			echo "yeah";
		}else{
			echo "no";
		}
	}
	if($mybb->settings['ultibackup_local'] == 0){
		unlink($destination);
	}
}
?>